﻿using System;
using System.Net;
using System.Net.Sockets;
using System.Windows.Forms;
using System.Collections.Generic;
using System.Text;

namespace Server
{
    public partial class Form1 : Form
    {
        delegate void AppendTextDelegate(Control ctrl, string s);
        IPAddress ipaddr;
        Socket mainsock;
        AppendTextDelegate _textAppender;
        const int BufferByteSize = 4096;
        bool _connected;
        
        public bool IsConnected
        {
            get { return _connected; }
        }

        public Form1()
        {
            InitializeComponent();

            //TCP&IP 소켓생성
            mainsock = new Socket(AddressFamily.InterNetwork, SocketType.Stream, ProtocolType.Tcp);

            _connected = true;
            OnConnected();

            _textAppender = new AppendTextDelegate(AppendText);
        }

        private void Form1_Load(object sender, EventArgs e)
        {
            IPHostEntry entry = Dns.GetHostEntry(Dns.GetHostName());

            foreach (IPAddress addr in entry.AddressList)
            {
                if (addr.AddressFamily == AddressFamily.InterNetwork)
                {
                    ipaddr = addr;
                    break;
                }
            }

            if (ipaddr == null)
            {
                ipaddr = IPAddress.Loopback;
            }

            addr_edit.Text = ipaddr.ToString();
        }

        private void btnend_Click(object sender, EventArgs e)
        {
            bool fireLostEvent = _connected == true;
            _connected = false;

            //form이 열리면서 tcp소켓이 생성되어 클라이언트와 연결하기전에 종료를 눌러도
            //아래 조건식에 해당하게됨. 연결유무확인후 close를 해야함.
            if (mainsock != null)
            {
                try
                {
                    //shutdown주석을 풀고 프로그램 실행시 에러메시지가 나옴.
                    //mainsock.Shutdown(SocketShutdown.Both);
                    mainsock.Close();
                    AppendText(content_edit, string.Format("서버가 종료되었습니다."));
                }
                catch (Exception ex)
                {
                    MessageBox.Show(ex.Message);
                }

                if (fireLostEvent == true)
                {
                    OnConnectionLost();
                }

                mainsock = null;

            }
            else
                AppendText(content_edit, string.Format("클라이언트와 연결없이 종료합니다."));
        }

        public event Action Connected;
        protected virtual void OnConnected()
        {
            AppendText(content_edit, string.Format("Connected"));

            if (Connected == null)
            {
                return;
            }

            Connected();
        }

        public event Action ConnectionLost;
        protected virtual void OnConnectionLost()
        {
            AppendText(content_edit, string.Format("Connection lost"));

            if (ConnectionLost == null)
            {
                return;
            }

            ConnectionLost();
        }











        private void start_Click(object sender, EventArgs e)
        {
            int port;
            if (!int.TryParse(portnum_edit.Text, out port))
            {
                AppendText(content_edit, string.Format("포트 번호가 잘못 입력되었거나 입력되지 않았습니다."));
                portnum_edit.Focus();
                portnum_edit.SelectAll();
                return;
            }

            IPEndPoint serverEP = new IPEndPoint(ipaddr, port);
            mainsock.Bind(serverEP);
            mainsock.Listen(5);
            mainsock.BeginAccept(AcceptCallback, null);
        }

        void AppendText(Control ctrl, string s)
        {
            if (ctrl.InvokeRequired) ctrl.Invoke(_textAppender, ctrl, s);
            else
            {
                string source = ctrl.Text;
                ctrl.Text = source + Environment.NewLine + s;
            }
        }

        void AcceptCallback(IAsyncResult ar)
        {
           
        }

        void DataReceived(IAsyncResult ar)
        {
            
        }
    }
}
